@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -71,7 +71,7 @@ static unsigned char tohex[16] = {
  *
  * Creates a new #GMimeParam node with name @name and value @value.
  *
- * Returns a new paramter structure.
+ * Returns: a new paramter structure.
  **/
 GMimeParam *
 g_mime_param_new (const char *name, const char *value)
@@ -141,11 +141,11 @@ decode_quoted_string (const char **in)
 	
 	if (*inptr == '"') {
 		start++;
-		out = g_strndup (start, (unsigned int) (inptr - start));
+		out = g_strndup (start, (size_t) (inptr - start));
 		inptr++;
 	} else {
 		/* string wasn't properly quoted */
-		out = g_strndup (start, (unsigned int) (inptr - start));
+		out = g_strndup (start, (size_t) (inptr - start));
 	}
 	
 	*in = inptr;
@@ -173,11 +173,24 @@ decode_token (const char **in)
 	decode_lwsp (&inptr);
 	
 	start = inptr;
+#ifdef STRICT_PARSER
 	while (is_ttoken (*inptr))
 		inptr++;
+#else
+	/* Broken mail clients like to make our lives difficult. Scan
+	 * for a ';' instead of trusting that the client followed the
+	 * specification. */
+	while (*inptr && *inptr != ';')
+		inptr++;
+	
+	/* Scan backwards over any trailing lwsp */
+	while (inptr > start && is_lwsp (inptr[-1]))
+		inptr--;
+#endif
+	
 	if (inptr > start) {
 		*in = inptr;
-		return g_strndup (start, (unsigned int) (inptr - start));
+		return g_strndup (start, (size_t) (inptr - start));
 	} else {
 		return NULL;
 	}
@@ -215,7 +228,7 @@ decode_param_token (const char **in)
 		inptr++;
 	if (inptr > start) {
 		*in = inptr;
-		return g_strndup (start, (unsigned int) (inptr - start));
+		return g_strndup (start, (size_t) (inptr - start));
 	} else {
 		return NULL;
 	}
@@ -396,7 +409,7 @@ rfc2184_param_charset (const char **in, char **langp)
 	
 	if (*inptr == '\'') {
 		if (langp)
-			*langp = g_strndup (lang, inptr - lang);
+			*langp = g_strndup (lang, (size_t) (inptr - lang));
 		
 		inptr++;
 	}
@@ -517,7 +530,8 @@ decode_param_list (const char *in)
 	char *name, *value;
 	gboolean encoded;
 	GString *gvalue;
-	int id, i;
+	guint i;
+	int id;
 	
 	params = NULL;
 	tail = (GMimeParam *) &params;
@@ -613,7 +627,7 @@ decode_param_list (const char *in)
  *
  * Creates a parameter list based on the input string.
  *
- * Returns a #GMimeParam structure based on @string.
+ * Returns: a #GMimeParam structure based on @string.
  **/
 GMimeParam *
 g_mime_param_new_from_string (const char *str)
@@ -646,6 +660,57 @@ g_mime_param_destroy (GMimeParam *param)
 }
 
 
+/**
+ * g_mime_param_next:
+ * @param: a #GMimeParam node
+ *
+ * Gets the next #GMimeParam node in the list.
+ *
+ * Returns: the next #GMimeParam node in the list.
+ **/
+const GMimeParam *
+g_mime_param_next (const GMimeParam *param)
+{
+	g_return_val_if_fail (param != NULL, NULL);
+	
+	return param->next;
+}
+
+
+/**
+ * g_mime_param_get_name:
+ * @param: a #GMimeParam
+ *
+ * Gets the name of the parameter.
+ *
+ * Returns: the name of the parameter.
+ **/
+const char *
+g_mime_param_get_name (const GMimeParam *param)
+{
+	g_return_val_if_fail (param != NULL, NULL);
+	
+	return param->name;
+}
+
+
+/**
+ * g_mime_param_get_value:
+ * @param: a #GMimeParam
+ *
+ * Gets the value of the parameter.
+ *
+ * Returns: the value of the parameter.
+ **/
+const char *
+g_mime_param_get_value (const GMimeParam *param)
+{
+	g_return_val_if_fail (param != NULL, NULL);
+	
+	return param->value;
+}
+
+
 /**
  * g_mime_param_append:
  * @params: param list
@@ -655,7 +720,7 @@ g_mime_param_destroy (GMimeParam *param)
  * Appends a new parameter with name @name and value @value to the
  * parameter list @params.
  *
- * Returns a param list with the new param of name @name and value
+ * Returns: a param list with the new param of name @name and value
  * @value appended to the list of params @params.
  **/
 GMimeParam *
@@ -686,7 +751,7 @@ g_mime_param_append (GMimeParam *params, const char *name, const char *value)
  *
  * Appends @param to the param list @params.
  *
- * Returns a param list with the new param @param appended to the list
+ * Returns: a param list with the new param @param appended to the list
  * of params @params.
  **/
 GMimeParam *
@@ -798,14 +863,14 @@ g_string_append_len_quoted (GString *out, const char *in, size_t len)
 }
 
 static void
-param_list_format (GString *out, GMimeParam *param, gboolean fold)
+param_list_format (GString *out, const GMimeParam *param, gboolean fold)
 {
 	int used = out->len;
 	
 	while (param) {
 		gboolean encoded = FALSE;
-		unsigned nlen, vlen;
 		int here = out->len;
+		size_t nlen, vlen;
 		int quote = 0;
 		char *value;
 		
@@ -844,7 +909,7 @@ param_list_format (GString *out, GMimeParam *param, gboolean fold)
 		
 		if (nlen + vlen + quote > GMIME_FOLD_LEN - 2) {
 			/* we need to do special rfc2184 parameter wrapping */
-			int maxlen = GMIME_FOLD_LEN - (nlen + 6);
+			size_t maxlen = GMIME_FOLD_LEN - (nlen + 6);
 			char *inptr, *inend;
 			int i = 0;
 			
@@ -852,7 +917,7 @@ param_list_format (GString *out, GMimeParam *param, gboolean fold)
 			inend = value + vlen;
 			
 			while (inptr < inend) {
-				char *ptr = inptr + MIN (inend - inptr, maxlen);
+				char *ptr = inptr + MIN ((size_t) (inend - inptr), maxlen);
 				
 				if (encoded && ptr < inend) {
 					/* be careful not to break an encoded char (ie %20) */
@@ -878,9 +943,9 @@ param_list_format (GString *out, GMimeParam *param, gboolean fold)
 							i++, encoded ? "*" : "");
 				
 				if (encoded || !quote)
-					g_string_append_len (out, inptr, ptr - inptr);
+					g_string_append_len (out, inptr, (size_t) (ptr - inptr));
 				else
-					g_string_append_len_quoted (out, inptr, ptr - inptr);
+					g_string_append_len_quoted (out, inptr, (size_t) (ptr - inptr));
 				
 				used += (out->len - here);
 				
@@ -919,7 +984,7 @@ param_list_format (GString *out, GMimeParam *param, gboolean fold)
  * Writes the params out to the string @string.
  **/
 void
-g_mime_param_write_to_string (GMimeParam *param, gboolean fold, GString *str)
+g_mime_param_write_to_string (const GMimeParam *param, gboolean fold, GString *str)
 {
 	g_return_if_fail (str != NULL);
 	
